/*********************************************************************
 * S_Socket.H -- simple AF_INET socket class
 * $Id: S_Socket.H,v 1.7 1998/08/04 15:35:03 tjko Exp $
 *
 * Copyright (c) 1998  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#ifndef _S_SOCKET_H_
#define _S_SOCKET_H_

#include <netinet/in.h>
#include <string.h>
#include <errno.h>

class S_Socket {
public:
  S_Socket();
  ~S_Socket();

  int setport(char const *service, char const *proto = "tcp");
  int setport(int port, char const *proto = "tcp");
  int setaddress(char const *address);
  int setaddress(unsigned int s_addr);

  int open(int nonblocking = 0);
  void close();

  int read(char *buf, int count);
  int write(char const *buf, int count);
  char *gets(char *s, int size);
  char *getstr(char *s, int size);
  int puts(char *s) { return write(s,strlen(s)); };

  
  int readyforread(int tmout = 0);
  int readyforwrite(int tmout = 0);

  int connected() const { return sock_open; };
  int port() const { return ntohs(portnum); };
  char const *portstr() const { return service_str; };
  char const *proto() const { return proto_str; };
  unsigned int address() const { return ntohl(saddr->s_addr); };
  char const *addresstr() const { return address_str; };
  char const *hostname();
  int fd() const { return fdes; };
  int error() const { return (int)err; };
  char const *errstr() const { return strerror(err); };
  
  // static class functions
  static void ignore_pipe();

private:
  int fdes;           /* socket file descriptor */
  int err;            /* last error code */
  int portnum;        /* port in network byteorder */
  int type;           /* socket type: SOCK_STREAM or SOCK_DGRAM  */
  int mode;           /* 0 or O_NONBLOCK */
  in_addr *saddr;
  sockaddr_in *saddress;
  int address_valid;
  int port_valid;
  int sock_open;
  char *address_str;
  char *service_str;
  char *proto_str;
  char *hostname_str;
#ifdef DEBUG
  void debug(char *format, ...);
#endif
};



#endif /* _S_SOCKET_H_ */
